>From 2d3146017d59211396b3a09716d938c9bece8fbe Mon Sep 17 00:00:00 2001
From: Michael Friedrich <michael.friedrich@univie.ac.at>
Date: Wed, 9 Feb 2011 17:58:55 +0100
Subject: [PATCH] core: fix race condition on flexible downtime commands when duration not set or zero

it may happen that an external command sets a
flexible downtime with duration not set or zero.

a flexible downtime is triggered between start and
end time and needs a duration to be set and non zero.

without the fix, the scheduled downtime starts and then
tries to calculate the end time out of the set duration
for the downtime and scheduling a new event.
the time passed over there is in the past, and causes the
core to get into a loop.

to prevent this misbehavior, a duration of null or zero is
not allowed and triggered as error in base/commands.c not
setting any flexible downtime.
---
 base/commands.c |   11 +++++++++++
 1 files changed, 11 insertions(+), 0 deletions(-)

diff --git a/base/commands.c b/base/commands.c
index 46e45e1..592ada4 100644
--- a/base/commands.c
+++ b/base/commands.c
@@ -2487,6 +2487,17 @@ int cmd_schedule_downtime(int cmd, time_t entry_time, char *args){
 	if((comment_data=my_strtok(NULL,";"))==NULL)
 		return ERROR;
 
+	/* check if flexible downtime demanded and duration set
+	   to non-zero.
+	   according to the documentation, a flexible downtime is
+	   started between start and end time and will last for
+	   "duration" seconds. strtoul converts a NULL value to 0
+	   so if set to 0, bail out as a duration>0 is needed. 	   */
+
+	if(fixed==0 && duration==0)
+		return ERROR;
+
+
 	/* duration should be auto-calculated, not user-specified */
 	if(fixed>0)
 		duration=(unsigned long)(end_time-start_time);
-- 
1.7.1.1

